@@ -87,6 +87,12 @@ namespace dxvk {
 
   using D3D9DeviceFlags = Flags<D3D9DeviceFlag>;
 
+  enum class D3D9DeviceLostState {
+    Ok = 0,
+    Lost = 1,
+    NotReset = 2,
+  };
+
   struct D3D9DrawInfo {
     uint32_t vertexCount;
     uint32_t instanceCount;
@@ -933,7 +939,7 @@ namespace dxvk {
     const D3D9ConstantLayout& GetVertexConstantLayout() { return m_vsLayout; }
     const D3D9ConstantLayout& GetPixelConstantLayout()  { return m_psLayout; }
 
-    HRESULT ResetState(D3DPRESENT_PARAMETERS* pPresentationParameters);
+    void ResetState(D3DPRESENT_PARAMETERS* pPresentationParameters);
     HRESULT ResetSwapChain(D3DPRESENT_PARAMETERS* pPresentationParameters, D3DDISPLAYMODEEX* pFullscreenDisplayMode);
 
     HRESULT InitialReset(D3DPRESENT_PARAMETERS* pPresentationParameters, D3DDISPLAYMODEEX* pFullscreenDisplayMode);
@@ -950,6 +956,43 @@ namespace dxvk {
     void TouchMappedTexture(D3D9CommonTexture* pTexture);
     void RemoveMappedTexture(D3D9CommonTexture* pTexture);
 
+    bool IsDeviceLost() const {
+      return m_deviceLostState != D3D9DeviceLostState::Ok;
+    }
+
+    void NotifyFullscreen(HWND window, bool fullscreen) {
+      D3D9DeviceLock lock = LockDevice();
+
+      if (fullscreen) {
+        if (unlikely(window != m_fullscreenWindow && m_fullscreenWindow != NULL)) {
+          Logger::warn("Multiple fullscreen windows detected.");
+        }
+        m_fullscreenWindow = window;
+      } else {
+        if (unlikely(m_fullscreenWindow != window)) {
+          Logger::warn("Window was not fullscreen in the first place.");
+        } else {
+          m_fullscreenWindow = 0;
+        }
+      }
+    }
+
+    void NotifyWindowActivated(HWND window, bool activated) {
+      D3D9DeviceLock lock = LockDevice();
+      
+      if (likely(!m_d3d9Options.deviceLost || IsExtended()))
+        return;
+
+      if (activated && m_deviceLostState == D3D9DeviceLostState::Lost) {
+        Logger::info("Device not reset");
+        m_deviceLostState = D3D9DeviceLostState::NotReset;
+      } else if (!activated && m_deviceLostState != D3D9DeviceLostState::Lost && m_fullscreenWindow == window) {
+        Logger::info("Device lost");
+        m_deviceLostState = D3D9DeviceLostState::Lost;
+        m_fullscreenWindow = NULL;
+      }
+    }
+
   private:
 
     DxvkCsChunkRef AllocCsChunk() {
@@ -1093,13 +1136,13 @@ namespace dxvk {
           return D3DERR_INVALIDCALL;
 
         if constexpr (ConstantType == D3D9ConstantType::Float) {
-          const float* source = set.fConsts[StartRegister].data;
+          const float* source = set->fConsts[StartRegister].data;
           const size_t size   = Count * sizeof(Vector4);
 
           std::memcpy(pConstantData, source, size);
         }
         else if constexpr (ConstantType == D3D9ConstantType::Int) {
-          const int*  source = set.iConsts[StartRegister].data;
+          const int*  source = set->iConsts[StartRegister].data;
           const size_t size  = Count * sizeof(Vector4i);
 
           std::memcpy(pConstantData, source, size);
@@ -1112,7 +1155,7 @@ namespace dxvk {
 
             const uint32_t bit         = (1u << bitIdx);
 
-            bool constValue = set.bConsts[arrayIdx] & bit;
+            bool constValue = set->bConsts[arrayIdx] & bit;
             pConstantData[i] = constValue ? TRUE : FALSE;
           }
         }
@@ -1315,12 +1358,17 @@ namespace dxvk {
     std::atomic<int64_t>            m_availableMemory = { 0 };
     std::atomic<int32_t>            m_samplerCount    = { 0 };
 
-    Direct3DState9                  m_state;
+    D3D9DeviceLostState             m_deviceLostState          = D3D9DeviceLostState::Ok;
+    HWND                            m_fullscreenWindow         = NULL;
 
 #ifdef D3D9_ALLOW_UNMAPPING
     lru_list<D3D9CommonTexture*>    m_mappedTextures;
 #endif
 
+    // m_state should be declared last (i.e. freed first), because it
+    // references objects that can call back into the device when freed.
+    Direct3DState9                  m_state;
+
     D3D9VkInteropDevice             m_d3d9Interop;
   };
 
