@@ -40,52 +40,20 @@
 
 // Missed definitions in Wine/MinGW.
 
-#ifndef D3DPRESENT_BACK_BUFFERS_MAX_EX
-#define D3DPRESENT_BACK_BUFFERS_MAX_EX 30
-#endif
-
-#ifndef D3DSI_OPCODE_MASK
-#define D3DSI_OPCODE_MASK 0x0000FFFF
-#endif
-
-#ifndef D3DSP_TEXTURETYPE_MASK
-#define D3DSP_TEXTURETYPE_MASK 0x78000000
-#endif
-
-#ifndef D3DUSAGE_AUTOGENMIPMAP
-#define D3DUSAGE_AUTOGENMIPMAP 0x00000400L
-#endif
-
-#ifndef D3DSP_DCL_USAGE_MASK
-#define D3DSP_DCL_USAGE_MASK 0x0000000f
-#endif
-
-#ifndef D3DSP_OPCODESPECIFICCONTROL_MASK
-#define D3DSP_OPCODESPECIFICCONTROL_MASK 0x00ff0000
-#endif
-
-#ifndef D3DSP_OPCODESPECIFICCONTROL_SHIFT
-#define D3DSP_OPCODESPECIFICCONTROL_SHIFT 16
-#endif
-
-#ifndef D3DCURSOR_IMMEDIATE_UPDATE
-#define D3DCURSOR_IMMEDIATE_UPDATE             0x00000001L
-#endif
-
 #ifndef D3DPRESENT_FORCEIMMEDIATE
 #define D3DPRESENT_FORCEIMMEDIATE              0x00000100L
 #endif
 
+#ifndef D3DSWAPEFFECT_COPY_VSYNC
+#define D3DSWAPEFFECT_COPY_VSYNC 4
+#endif
+
 // MinGW headers are broken. Who'dve guessed?
 #ifndef _MSC_VER
 typedef struct _D3DDEVINFO_RESOURCEMANAGER
 {
   char dummy;
 } D3DDEVINFO_RESOURCEMANAGER, * LPD3DDEVINFO_RESOURCEMANAGER;
-
-#ifndef __WINE__
-extern "C" WINUSERAPI WINBOOL WINAPI SetProcessDPIAware(VOID);
-#endif
 #endif
 
 // This is the managed pool on D3D9Ex, it's just hidden!
