@@ -6,13 +6,24 @@
 #include "dxvk_instance.h"
 
 namespace dxvk {
-  
+
+  DxvkDeviceQueue getDeviceQueue(const Rc<vk::DeviceFn>& vkd, uint32_t family, uint32_t index) {
+    DxvkDeviceQueue result = { };
+    result.queueFamily = family;
+    result.queueIndex = index;
+
+    if (family != VK_QUEUE_FAMILY_IGNORED)
+      vkd->vkGetDeviceQueue(vkd->device(), family, index, &result.queueHandle);
+
+    return result;
+  }
+
+
   DxvkAdapter::DxvkAdapter(
     const Rc<vk::InstanceFn>& vki,
           VkPhysicalDevice    handle)
   : m_vki           (vki),
     m_handle        (handle) {
-    this->initHeapAllocInfo();
     this->queryExtensions();
     this->queryDeviceInfo();
     this->queryDeviceFeatures();
@@ -39,13 +50,19 @@ namespace dxvk {
 
     for (uint32_t i = 0; i < info.heapCount; i++) {
       info.heaps[i].heapFlags = memProps.memoryProperties.memoryHeaps[i].flags;
+      info.heaps[i].heapSize = memProps.memoryProperties.memoryHeaps[i].size;
 
       if (m_hasMemoryBudget) {
+        // Handle DXVK's memory allocations separately so that
+        // freeing  resources actually is visible to applications.
+        VkDeviceSize allocated = m_memoryAllocated[i].load();
+        VkDeviceSize used = m_memoryUsed[i].load();
+
         info.heaps[i].memoryBudget    = memBudget.heapBudget[i];
-        info.heaps[i].memoryAllocated = memBudget.heapUsage[i];
+        info.heaps[i].memoryAllocated = std::max(memBudget.heapUsage[i], allocated) - allocated + used;
       } else {
         info.heaps[i].memoryBudget    = memProps.memoryProperties.memoryHeaps[i].size;
-        info.heaps[i].memoryAllocated = m_heapAlloc[i].load();
+        info.heaps[i].memoryAllocated = m_memoryUsed[i].load();
       }
     }
 
@@ -338,35 +355,7 @@ namespace dxvk {
     const Rc<DxvkInstance>&   instance,
           DxvkDeviceFeatures  enabledFeatures) {
     DxvkDeviceExtensions devExtensions;
-
-    std::array<DxvkExt*, 26> devExtensionList = {{
-      &devExtensions.amdMemoryOverallocationBehaviour,
-      &devExtensions.amdShaderFragmentMask,
-      &devExtensions.extAttachmentFeedbackLoopLayout,
-      &devExtensions.extConservativeRasterization,
-      &devExtensions.extCustomBorderColor,
-      &devExtensions.extDepthClipEnable,
-      &devExtensions.extExtendedDynamicState3,
-      &devExtensions.extFragmentShaderInterlock,
-      &devExtensions.extFullScreenExclusive,
-      &devExtensions.extGraphicsPipelineLibrary,
-      &devExtensions.extHdrMetadata,
-      &devExtensions.extMemoryBudget,
-      &devExtensions.extMemoryPriority,
-      &devExtensions.extNonSeamlessCubeMap,
-      &devExtensions.extRobustness2,
-      &devExtensions.extShaderModuleIdentifier,
-      &devExtensions.extShaderStencilExport,
-      &devExtensions.extSwapchainColorSpace,
-      &devExtensions.extTransformFeedback,
-      &devExtensions.extVertexAttributeDivisor,
-      &devExtensions.khrExternalMemoryWin32,
-      &devExtensions.khrExternalSemaphoreWin32,
-      &devExtensions.khrPipelineLibrary,
-      &devExtensions.khrSwapchain,
-      &devExtensions.nvxBinaryImport,
-      &devExtensions.nvxImageViewHandle,
-    }};
+    auto devExtensionList = getExtensionList(devExtensions);
 
     // Only enable Cuda interop extensions in 64-bit builds in
     // order to avoid potential driver or address space issues.
@@ -388,7 +377,7 @@ namespace dxvk {
     if (!m_deviceExtensions.enableExtensions(
           devExtensionList.size(),
           devExtensionList.data(),
-          extensionsEnabled))
+          &extensionsEnabled))
       throw DxvkError("DxvkAdapter: Failed to create device");
     
     // Enable additional extensions if necessary
@@ -472,118 +461,15 @@ namespace dxvk {
       m_deviceFeatures.extShaderModuleIdentifier.shaderModuleIdentifier;
 
     // Create pNext chain for additional device features
-    enabledFeatures.core.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FEATURES_2_KHR;
-    enabledFeatures.core.pNext = nullptr;
-
-    enabledFeatures.vk11.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_1_FEATURES;
-    enabledFeatures.vk11.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.vk11);
-
-    enabledFeatures.vk12.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_2_FEATURES;
-    enabledFeatures.vk12.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.vk12);
-
-    enabledFeatures.vk13.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_3_FEATURES;
-    enabledFeatures.vk13.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.vk13);
-
-    if (devExtensions.amdShaderFragmentMask)
-      enabledFeatures.amdShaderFragmentMask = VK_TRUE;
-
-    if (devExtensions.extAttachmentFeedbackLoopLayout) {
-      enabledFeatures.extAttachmentFeedbackLoopLayout.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ATTACHMENT_FEEDBACK_LOOP_LAYOUT_FEATURES_EXT;
-      enabledFeatures.extAttachmentFeedbackLoopLayout.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extAttachmentFeedbackLoopLayout);
-    }
-
-    if (devExtensions.extConservativeRasterization)
-      enabledFeatures.extConservativeRasterization = VK_TRUE;
-
-    if (devExtensions.extCustomBorderColor) {
-      enabledFeatures.extCustomBorderColor.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_CUSTOM_BORDER_COLOR_FEATURES_EXT;
-      enabledFeatures.extCustomBorderColor.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extCustomBorderColor);
-    }
-
-    if (devExtensions.extDepthClipEnable) {
-      enabledFeatures.extDepthClipEnable.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DEPTH_CLIP_ENABLE_FEATURES_EXT;
-      enabledFeatures.extDepthClipEnable.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extDepthClipEnable);
-    }
-
-    if (devExtensions.extExtendedDynamicState3) {
-      enabledFeatures.extExtendedDynamicState3.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTENDED_DYNAMIC_STATE_3_FEATURES_EXT;
-      enabledFeatures.extExtendedDynamicState3.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extExtendedDynamicState3);
-    }
-
-    if (devExtensions.extFragmentShaderInterlock) {
-      enabledFeatures.extFragmentShaderInterlock.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_SHADER_INTERLOCK_FEATURES_EXT;
-      enabledFeatures.extFragmentShaderInterlock.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extFragmentShaderInterlock);
-    }
-
-    if (devExtensions.extFullScreenExclusive)
-      enabledFeatures.extFullScreenExclusive = VK_TRUE;
-
-    if (devExtensions.extGraphicsPipelineLibrary) {
-      enabledFeatures.extGraphicsPipelineLibrary.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_GRAPHICS_PIPELINE_LIBRARY_FEATURES_EXT;
-      enabledFeatures.extGraphicsPipelineLibrary.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extGraphicsPipelineLibrary);
-    }
-
-    if (devExtensions.extMemoryBudget)
-      enabledFeatures.extMemoryBudget = VK_TRUE;
-
-    if (devExtensions.extMemoryPriority) {
-      enabledFeatures.extMemoryPriority.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MEMORY_PRIORITY_FEATURES_EXT;
-      enabledFeatures.extMemoryPriority.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extMemoryPriority);
-    }
-
-    if (devExtensions.extNonSeamlessCubeMap) {
-      enabledFeatures.extNonSeamlessCubeMap.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_NON_SEAMLESS_CUBE_MAP_FEATURES_EXT;
-      enabledFeatures.extNonSeamlessCubeMap.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extNonSeamlessCubeMap);
-    }
-
-    if (devExtensions.extShaderModuleIdentifier) {
-      enabledFeatures.extShaderModuleIdentifier.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_MODULE_IDENTIFIER_FEATURES_EXT;
-      enabledFeatures.extShaderModuleIdentifier.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extShaderModuleIdentifier);
-    }
-
-    if (devExtensions.extRobustness2) {
-      enabledFeatures.extRobustness2.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ROBUSTNESS_2_FEATURES_EXT;
-      enabledFeatures.extRobustness2.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extRobustness2);
-    }
-
-    if (devExtensions.extShaderStencilExport)
-      enabledFeatures.extShaderStencilExport = VK_TRUE;
-
-    if (devExtensions.extSwapchainColorSpace)
-      enabledFeatures.extSwapchainColorSpace = VK_TRUE;
-
-    if (devExtensions.extHdrMetadata)
-      enabledFeatures.extHdrMetadata = VK_TRUE;
-
-    if (devExtensions.extTransformFeedback) {
-      enabledFeatures.extTransformFeedback.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TRANSFORM_FEEDBACK_FEATURES_EXT;
-      enabledFeatures.extTransformFeedback.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extTransformFeedback);
-    }
-
-    if (devExtensions.extVertexAttributeDivisor.revision() >= 3) {
-      enabledFeatures.extVertexAttributeDivisor.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VERTEX_ATTRIBUTE_DIVISOR_FEATURES_EXT;
-      enabledFeatures.extVertexAttributeDivisor.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extVertexAttributeDivisor);
-    }
-
-    if (devExtensions.khrExternalMemoryWin32)
-      enabledFeatures.khrExternalMemoryWin32 = VK_TRUE;
-
-    if (devExtensions.khrExternalSemaphoreWin32)
-      enabledFeatures.khrExternalSemaphoreWin32 = VK_TRUE;
-
-    if (devExtensions.nvxBinaryImport)
-      enabledFeatures.nvxBinaryImport = VK_TRUE;
-
-    if (devExtensions.nvxImageViewHandle)
-      enabledFeatures.nvxImageViewHandle = VK_TRUE;
+    initFeatureChain(enabledFeatures, devExtensions, instance->extensions());
 
     // Log feature support info an extension list
     Logger::info(str::format("Device properties:"
-      "\n  Device name:     : ", m_deviceInfo.core.properties.deviceName,
-      "\n  Driver version   : ",
-        VK_VERSION_MAJOR(m_deviceInfo.core.properties.driverVersion), ".",
-        VK_VERSION_MINOR(m_deviceInfo.core.properties.driverVersion), ".",
-        VK_VERSION_PATCH(m_deviceInfo.core.properties.driverVersion)));
+      "\n  Device : ", m_deviceInfo.core.properties.deviceName,
+      "\n  Driver : ", m_deviceInfo.vk12.driverName, " ",
+      VK_VERSION_MAJOR(m_deviceInfo.core.properties.driverVersion), ".",
+      VK_VERSION_MINOR(m_deviceInfo.core.properties.driverVersion), ".",
+      VK_VERSION_PATCH(m_deviceInfo.core.properties.driverVersion)));
 
     Logger::info("Enabled device extensions:");
     this->logNameList(extensionNameList);
@@ -650,25 +536,142 @@ namespace dxvk {
     if (vr != VK_SUCCESS)
       throw DxvkError("DxvkAdapter: Failed to create device");
     
-    return new DxvkDevice(instance, this,
-      new vk::DeviceFn(m_vki, true, device),
-      enabledFeatures);
-  }
-  
-  
-  void DxvkAdapter::notifyHeapMemoryAlloc(
-          uint32_t            heap,
-          VkDeviceSize        bytes) {
-    if (!m_hasMemoryBudget)
-      m_heapAlloc[heap] += bytes;
+    Rc<vk::DeviceFn> vkd = new vk::DeviceFn(m_vki, true, device);
+
+    DxvkDeviceQueueSet queues = { };
+    queues.graphics = getDeviceQueue(vkd, queueFamilies.graphics, 0);
+    queues.transfer = getDeviceQueue(vkd, queueFamilies.transfer, 0);
+    queues.sparse = getDeviceQueue(vkd, queueFamilies.sparse, 0);
+
+    return new DxvkDevice(instance, this, vkd, enabledFeatures, queues, DxvkQueueCallback());
   }
 
-  
-  void DxvkAdapter::notifyHeapMemoryFree(
+
+  Rc<DxvkDevice> DxvkAdapter::importDevice(
+    const Rc<DxvkInstance>&   instance,
+    const DxvkDeviceImportInfo& args) {
+    DxvkDeviceExtensions devExtensions;
+    auto devExtensionList = getExtensionList(devExtensions);
+
+    if (!m_deviceExtensions.enableExtensions(devExtensionList.size(), devExtensionList.data(), nullptr))
+      throw DxvkError("DxvkAdapter: Failed to create device");
+    
+    DxvkNameList extensionNameList(args.extensionCount, args.extensionNames);
+
+    // Populate feature structs based on imported Vulkan device
+    DxvkDeviceFeatures enabledFeatures = { };
+
+    for (auto f = reinterpret_cast<const VkBaseOutStructure*>(args.features); f; f = f->pNext) {
+      switch (f->sType) {
+        case VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FEATURES_2:
+          enabledFeatures.core.features = reinterpret_cast<const VkPhysicalDeviceFeatures2*>(f)->features;
+          break;
+
+        case VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_1_FEATURES:
+          enabledFeatures.vk11 = *reinterpret_cast<const VkPhysicalDeviceVulkan11Features*>(f);
+          break;
+
+        case VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_2_FEATURES:
+          enabledFeatures.vk12 = *reinterpret_cast<const VkPhysicalDeviceVulkan12Features*>(f);
+          break;
+
+        case VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_3_FEATURES:
+          enabledFeatures.vk13 = *reinterpret_cast<const VkPhysicalDeviceVulkan13Features*>(f);
+          break;
+
+        case VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ATTACHMENT_FEEDBACK_LOOP_LAYOUT_FEATURES_EXT:
+          enabledFeatures.extAttachmentFeedbackLoopLayout = *reinterpret_cast<const VkPhysicalDeviceAttachmentFeedbackLoopLayoutFeaturesEXT*>(f);
+          break;
+
+        case VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_CUSTOM_BORDER_COLOR_FEATURES_EXT:
+          enabledFeatures.extCustomBorderColor = *reinterpret_cast<const VkPhysicalDeviceCustomBorderColorFeaturesEXT*>(f);
+          break;
+
+        case VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DEPTH_CLIP_ENABLE_FEATURES_EXT:
+          enabledFeatures.extDepthClipEnable = *reinterpret_cast<const VkPhysicalDeviceDepthClipEnableFeaturesEXT*>(f);
+          break;
+
+        case VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTENDED_DYNAMIC_STATE_3_FEATURES_EXT:
+          enabledFeatures.extExtendedDynamicState3 = *reinterpret_cast<const VkPhysicalDeviceExtendedDynamicState3FeaturesEXT*>(f);
+          break;
+
+        case VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_SHADER_INTERLOCK_FEATURES_EXT:
+          enabledFeatures.extFragmentShaderInterlock = *reinterpret_cast<const VkPhysicalDeviceFragmentShaderInterlockFeaturesEXT*>(f);
+          break;
+
+        case VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_GRAPHICS_PIPELINE_LIBRARY_FEATURES_EXT:
+          enabledFeatures.extGraphicsPipelineLibrary = *reinterpret_cast<const VkPhysicalDeviceGraphicsPipelineLibraryFeaturesEXT*>(f);
+          break;
+
+        case VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MEMORY_PRIORITY_FEATURES_EXT:
+          enabledFeatures.extMemoryPriority = *reinterpret_cast<const VkPhysicalDeviceMemoryPriorityFeaturesEXT*>(f);
+          break;
+
+        case VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_NON_SEAMLESS_CUBE_MAP_FEATURES_EXT:
+          enabledFeatures.extNonSeamlessCubeMap = *reinterpret_cast<const VkPhysicalDeviceNonSeamlessCubeMapFeaturesEXT*>(f);
+          break;
+
+        case VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ROBUSTNESS_2_FEATURES_EXT:
+          enabledFeatures.extRobustness2 = *reinterpret_cast<const VkPhysicalDeviceRobustness2FeaturesEXT*>(f);
+          break;
+
+        case VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_MODULE_IDENTIFIER_FEATURES_EXT:
+          enabledFeatures.extShaderModuleIdentifier = *reinterpret_cast<const VkPhysicalDeviceShaderModuleIdentifierFeaturesEXT*>(f);
+          break;
+
+        case VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TRANSFORM_FEEDBACK_FEATURES_EXT:
+          enabledFeatures.extTransformFeedback = *reinterpret_cast<const VkPhysicalDeviceTransformFeedbackFeaturesEXT*>(f);
+          break;
+
+        case VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VERTEX_ATTRIBUTE_DIVISOR_FEATURES_EXT:
+          enabledFeatures.extVertexAttributeDivisor = *reinterpret_cast<const VkPhysicalDeviceVertexAttributeDivisorFeaturesEXT*>(f);
+          break;
+
+        default:
+          // Ignore any unknown feature structs
+          break;
+      }
+    }
+
+    initFeatureChain(enabledFeatures, devExtensions, instance->extensions());
+
+    // Log feature support info an extension list
+    Logger::info(str::format("Device properties:"
+      "\n  Device name: ", m_deviceInfo.core.properties.deviceName,
+      "\n  Driver:      ", m_deviceInfo.vk12.driverName, " ",
+      VK_VERSION_MAJOR(m_deviceInfo.core.properties.driverVersion), ".",
+      VK_VERSION_MINOR(m_deviceInfo.core.properties.driverVersion), ".",
+      VK_VERSION_PATCH(m_deviceInfo.core.properties.driverVersion)));
+
+    Logger::info("Enabled device extensions:");
+    this->logNameList(extensionNameList);
+    this->logFeatures(enabledFeatures);
+
+    // Create device loader
+    Rc<vk::DeviceFn> vkd = new vk::DeviceFn(m_vki, false, args.device);
+
+    // We only support one queue when importing devices, and no sparse.
+    DxvkDeviceQueueSet queues = { };
+    queues.graphics = { args.queue, args.queueFamily };
+    queues.transfer = queues.graphics;
+
+    return new DxvkDevice(instance, this, vkd, enabledFeatures, queues, args.queueCallback);
+  }
+
+
+  void DxvkAdapter::notifyMemoryAlloc(
           uint32_t            heap,
-          VkDeviceSize        bytes) {
-    if (!m_hasMemoryBudget)
-      m_heapAlloc[heap] -= bytes;
+          int64_t             bytes) {
+    if (heap < m_memoryAllocated.size())
+      m_memoryAllocated[heap] += bytes;
+  }
+
+
+  void DxvkAdapter::notifyMemoryUse(
+          uint32_t            heap,
+          int64_t             bytes) {
+    if (heap < m_memoryUsed.size())
+      m_memoryUsed[heap] += bytes;
   }
 
 
@@ -686,18 +689,14 @@ namespace dxvk {
   
   
   void DxvkAdapter::logAdapterInfo() const {
-    VkPhysicalDeviceProperties deviceInfo = this->deviceProperties();
-    VkPhysicalDeviceMemoryProperties memoryInfo = this->memoryProperties();
+    const auto deviceInfo = this->devicePropertiesExt();
+    const auto memoryInfo = this->memoryProperties();
     
-    Logger::info(str::format(deviceInfo.deviceName, ":"));
-    Logger::info(str::format("  Driver: ",
-      VK_VERSION_MAJOR(deviceInfo.driverVersion), ".",
-      VK_VERSION_MINOR(deviceInfo.driverVersion), ".",
-      VK_VERSION_PATCH(deviceInfo.driverVersion)));
-    Logger::info(str::format("  Vulkan: ",
-      VK_VERSION_MAJOR(deviceInfo.apiVersion), ".",
-      VK_VERSION_MINOR(deviceInfo.apiVersion), ".",
-      VK_VERSION_PATCH(deviceInfo.apiVersion)));
+    Logger::info(str::format(deviceInfo.core.properties.deviceName, ":",
+      "\n  Driver : ", deviceInfo.vk12.driverName, " ",
+      VK_VERSION_MAJOR(deviceInfo.core.properties.driverVersion), ".",
+      VK_VERSION_MINOR(deviceInfo.core.properties.driverVersion), ".",
+      VK_VERSION_PATCH(deviceInfo.core.properties.driverVersion)));
 
     for (uint32_t i = 0; i < memoryInfo.memoryHeapCount; i++) {
       constexpr VkDeviceSize mib = 1024 * 1024;
@@ -728,12 +727,6 @@ namespace dxvk {
   }
   
   
-  void DxvkAdapter::initHeapAllocInfo() {
-    for (uint32_t i = 0; i < m_heapAlloc.size(); i++)
-      m_heapAlloc[i] = 0;
-  }
-
-
   void DxvkAdapter::queryExtensions() {
     m_deviceExtensions = DxvkNameSet::enumDeviceExtensions(m_vki, m_handle);
   }
@@ -946,7 +939,151 @@ namespace dxvk {
 
     return VK_QUEUE_FAMILY_IGNORED;
   }
-  
+
+
+  std::vector<DxvkExt*> DxvkAdapter::getExtensionList(
+          DxvkDeviceExtensions&   devExtensions) {
+    return {{
+      &devExtensions.amdMemoryOverallocationBehaviour,
+      &devExtensions.amdShaderFragmentMask,
+      &devExtensions.extAttachmentFeedbackLoopLayout,
+      &devExtensions.extConservativeRasterization,
+      &devExtensions.extCustomBorderColor,
+      &devExtensions.extDepthClipEnable,
+      &devExtensions.extExtendedDynamicState3,
+      &devExtensions.extFragmentShaderInterlock,
+      &devExtensions.extFullScreenExclusive,
+      &devExtensions.extGraphicsPipelineLibrary,
+      &devExtensions.extHdrMetadata,
+      &devExtensions.extMemoryBudget,
+      &devExtensions.extMemoryPriority,
+      &devExtensions.extNonSeamlessCubeMap,
+      &devExtensions.extRobustness2,
+      &devExtensions.extShaderModuleIdentifier,
+      &devExtensions.extShaderStencilExport,
+      &devExtensions.extSwapchainColorSpace,
+      &devExtensions.extTransformFeedback,
+      &devExtensions.extVertexAttributeDivisor,
+      &devExtensions.khrExternalMemoryWin32,
+      &devExtensions.khrExternalSemaphoreWin32,
+      &devExtensions.khrPipelineLibrary,
+      &devExtensions.khrSwapchain,
+      &devExtensions.nvxBinaryImport,
+      &devExtensions.nvxImageViewHandle,
+    }};
+  }
+
+
+  void DxvkAdapter::initFeatureChain(
+          DxvkDeviceFeatures&   enabledFeatures,
+    const DxvkDeviceExtensions& devExtensions,
+    const DxvkInstanceExtensions& insExtensions) {
+    enabledFeatures.core.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FEATURES_2_KHR;
+    enabledFeatures.core.pNext = nullptr;
+
+    enabledFeatures.vk11.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_1_FEATURES;
+    enabledFeatures.vk11.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.vk11);
+
+    enabledFeatures.vk12.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_2_FEATURES;
+    enabledFeatures.vk12.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.vk12);
+
+    enabledFeatures.vk13.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_3_FEATURES;
+    enabledFeatures.vk13.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.vk13);
+
+    if (devExtensions.amdShaderFragmentMask)
+      enabledFeatures.amdShaderFragmentMask = VK_TRUE;
+
+    if (devExtensions.extAttachmentFeedbackLoopLayout) {
+      enabledFeatures.extAttachmentFeedbackLoopLayout.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ATTACHMENT_FEEDBACK_LOOP_LAYOUT_FEATURES_EXT;
+      enabledFeatures.extAttachmentFeedbackLoopLayout.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extAttachmentFeedbackLoopLayout);
+    }
+
+    if (devExtensions.extConservativeRasterization)
+      enabledFeatures.extConservativeRasterization = VK_TRUE;
+
+    if (devExtensions.extCustomBorderColor) {
+      enabledFeatures.extCustomBorderColor.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_CUSTOM_BORDER_COLOR_FEATURES_EXT;
+      enabledFeatures.extCustomBorderColor.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extCustomBorderColor);
+    }
+
+    if (devExtensions.extDepthClipEnable) {
+      enabledFeatures.extDepthClipEnable.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DEPTH_CLIP_ENABLE_FEATURES_EXT;
+      enabledFeatures.extDepthClipEnable.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extDepthClipEnable);
+    }
+
+    if (devExtensions.extExtendedDynamicState3) {
+      enabledFeatures.extExtendedDynamicState3.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTENDED_DYNAMIC_STATE_3_FEATURES_EXT;
+      enabledFeatures.extExtendedDynamicState3.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extExtendedDynamicState3);
+    }
+
+    if (devExtensions.extFragmentShaderInterlock) {
+      enabledFeatures.extFragmentShaderInterlock.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_SHADER_INTERLOCK_FEATURES_EXT;
+      enabledFeatures.extFragmentShaderInterlock.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extFragmentShaderInterlock);
+    }
+
+    if (devExtensions.extFullScreenExclusive && insExtensions.khrGetSurfaceCapabilities2)
+      enabledFeatures.extFullScreenExclusive = VK_TRUE;
+
+    if (devExtensions.extGraphicsPipelineLibrary) {
+      enabledFeatures.extGraphicsPipelineLibrary.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_GRAPHICS_PIPELINE_LIBRARY_FEATURES_EXT;
+      enabledFeatures.extGraphicsPipelineLibrary.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extGraphicsPipelineLibrary);
+    }
+
+    if (devExtensions.extMemoryBudget)
+      enabledFeatures.extMemoryBudget = VK_TRUE;
+
+    if (devExtensions.extMemoryPriority) {
+      enabledFeatures.extMemoryPriority.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MEMORY_PRIORITY_FEATURES_EXT;
+      enabledFeatures.extMemoryPriority.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extMemoryPriority);
+    }
+
+    if (devExtensions.extNonSeamlessCubeMap) {
+      enabledFeatures.extNonSeamlessCubeMap.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_NON_SEAMLESS_CUBE_MAP_FEATURES_EXT;
+      enabledFeatures.extNonSeamlessCubeMap.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extNonSeamlessCubeMap);
+    }
+
+    if (devExtensions.extShaderModuleIdentifier) {
+      enabledFeatures.extShaderModuleIdentifier.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_MODULE_IDENTIFIER_FEATURES_EXT;
+      enabledFeatures.extShaderModuleIdentifier.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extShaderModuleIdentifier);
+    }
+
+    if (devExtensions.extRobustness2) {
+      enabledFeatures.extRobustness2.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ROBUSTNESS_2_FEATURES_EXT;
+      enabledFeatures.extRobustness2.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extRobustness2);
+    }
+
+    if (devExtensions.extShaderStencilExport)
+      enabledFeatures.extShaderStencilExport = VK_TRUE;
+
+    if (devExtensions.extSwapchainColorSpace)
+      enabledFeatures.extSwapchainColorSpace = VK_TRUE;
+
+    if (devExtensions.extHdrMetadata)
+      enabledFeatures.extHdrMetadata = VK_TRUE;
+
+    if (devExtensions.extTransformFeedback) {
+      enabledFeatures.extTransformFeedback.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TRANSFORM_FEEDBACK_FEATURES_EXT;
+      enabledFeatures.extTransformFeedback.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extTransformFeedback);
+    }
+
+    if (devExtensions.extVertexAttributeDivisor.revision() >= 3) {
+      enabledFeatures.extVertexAttributeDivisor.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VERTEX_ATTRIBUTE_DIVISOR_FEATURES_EXT;
+      enabledFeatures.extVertexAttributeDivisor.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extVertexAttributeDivisor);
+    }
+
+    if (devExtensions.khrExternalMemoryWin32)
+      enabledFeatures.khrExternalMemoryWin32 = VK_TRUE;
+
+    if (devExtensions.khrExternalSemaphoreWin32)
+      enabledFeatures.khrExternalSemaphoreWin32 = VK_TRUE;
+
+    if (devExtensions.nvxBinaryImport)
+      enabledFeatures.nvxBinaryImport = VK_TRUE;
+
+    if (devExtensions.nvxImageViewHandle)
+      enabledFeatures.nvxImageViewHandle = VK_TRUE;
+  }
+
   
   void DxvkAdapter::logNameList(const DxvkNameList& names) {
     for (uint32_t i = 0; i < names.count(); i++)
