@@ -43,6 +43,14 @@ namespace dxvk {
       { "d3d11.dcSingleUseMode",            "False" },
       { "d3d11.cachedDynamicResources",     "vi"   },
     }} },
+    /* Far Cry 2: Set vendor ID to Nvidia to avoid
+     * vegetation artifacts on Intel, and set
+     * apitrace mode to True to improve perf on all
+     * hardware.                                  */
+    { R"(\\(FarCry2|farcry2game)\.exe$)", {{
+      { "d3d9.customVendorId",              "10de" },
+      { "d3d9.apitraceMode",                "True" },
+    }} },
     /* Far Cry 3: Assumes clear(0.5) on an UNORM  *
      * format to result in 128 on AMD and 127 on  *
      * Nvidia. We assume that the Vulkan drivers  *
@@ -60,9 +68,9 @@ namespace dxvk {
       { "dxgi.deferSurfaceCreation",        "True" },
       { "d3d11.cachedDynamicResources",     "c" },
     }} },
-    /* Nioh: See Frostpunk, apparently?           */
+    /* Nioh: Apparently the same as the Atelier games  */
     { R"(\\nioh\.exe$)", {{
-      { "dxgi.deferSurfaceCreation",        "True" },
+      { "d3d9.deferSurfaceCreation",        "True" },
     }} },
     /* Quantum Break: Mever initializes shared    *
      * memory in one of its compute shaders.      *
@@ -131,6 +139,11 @@ namespace dxvk {
     { R"(\\h1(_[ms]p64_ship|-mod)\.exe$)", {{
       { "dxgi.customVendorId",              "10de" },
     }} },
+    /* Modern Warfare 2 Campaign Remastered       *
+     * AMD AGS crash same as above                */
+    { R"(\\MW2CR.exe\.exe$)", {{
+      { "dxgi.customVendorId",              "10de" },
+    }} },
     /* Crysis 3 - slower if it notices AMD card     *
      * Apitrace mode helps massively in cpu bound   *
      * game parts                                   */
@@ -159,7 +172,7 @@ namespace dxvk {
       { "d3d9.deferSurfaceCreation",        "True" },
     }} },
     /* Just how many of these games are there?    */
-    { R"(\\Atelier_(Lulua|Lydie_and_Suelle|Ryza(_2)?|Sophie_2)\.exe$)", {{
+    { R"(\\Atelier_(Lulua|Lydie_and_Suelle|Ryza(_2|_3)?|Sophie_2)\.exe$)", {{
       { "d3d9.deferSurfaceCreation",        "True" },
     }} },
     /* ...                                        */
@@ -329,6 +342,24 @@ namespace dxvk {
     { R"(\\RapaNui-Win64-Shipping\.exe$)", {{
       { "dxgi.customVendorId",              "8086" },
     }} },
+    /* SpellForce 3 Reforced & expansions         *
+     * Greatly improves CPU bound performance     */
+    { R"(\\SF3ClientFinal\.exe$)", {{
+      { "d3d11.cachedDynamicResources",        "v" },
+    }} },
+    /* Tom Clancy's Ghost Recon Breakpoint        */
+    { R"(\\GRB\.exe$)", {{
+      { "dxgi.nvapiHack",                  "False" },
+    }} },
+    /* GTA V performance issues                   */
+    { R"(\\GTA5\.exe$)", {{
+      { "d3d11.cachedDynamicResources",     "vi"   },
+    }} },
+    /* Crash Bandicoot N. Sane Trilogy            *
+     * Work around some vsync funkiness           */
+    { R"(\\CrashBandicootNSaneTrilogy\.exe$)", {{
+      { "dxgi.syncInterval",                "1"   },
+    }} },
 
     /**********************************************/
     /* D3D9 GAMES                                 */
@@ -435,20 +466,18 @@ namespace dxvk {
     { R"(\\ToEE(a)?\.exe$)", {{
       { "d3d9.allowDiscard",                "False" },
     }} },
-    /* ZUSI 3 - Aerosoft Edition                  */
-    { R"(\\ZusiSim\.exe$)", {{
-      { "d3d9.noExplicitFrontBuffer",       "True" },
-    }} },
     /* GTA IV (NVAPI)                             */
     /* Also thinks we're always on Intel          *
      * and will report/use bad amounts of VRAM.
      * Disabling support for DF texture formats
      * makes the game use a better looking render
-     * path for mirrors                           */
+     * path for mirrors.
+     * Also runs into issues after alt-tabbing.   */
     { R"(\\(GTAIV|EFLC)\.exe$)", {{
       { "d3d9.customVendorId",              "1002" },
       { "dxgi.emulateUMA",                  "True" },
       { "d3d9.supportDFFormats",            "False" },
+      { "d3d9.deviceLost",                  "True" },
     }} },
     /* Battlefield 2 (bad z-pass)                 */
     { R"(\\BF2\.exe$)", {{
@@ -513,10 +542,6 @@ namespace dxvk {
     { R"(\\SineMoraEX\.exe$)", {{
       { "d3d9.maxFrameRate",                "60" },
     }} },
-    /* Fantasy Grounds                           */
-    { R"(\\FantasyGrounds\.exe$)", {{
-      { "d3d9.noExplicitFrontBuffer",       "True" },
-    }} },
     /* Red Orchestra 2                           */
     { R"(\\ROGame\.exe$)", {{
       { "d3d9.floatEmulation",              "Strict" },
@@ -557,17 +582,6 @@ namespace dxvk {
     { R"(\\limbo\.exe$)", {{
       { "d3d9.maxFrameRate",                "60" },
     }} },
-    /* Warhammer: Return of Reckoning Launcher
-       Forcing SM1 fixes a black window otherwise caused by
-       the lack of support for partial presentation */
-    { R"(\\RoRLauncher\.exe$)", {{
-      { "d3d9.shaderModel",                 "1" },
-    }} },
-    /* Halo CE SPV3 launcher
-       Same issue as Warhammer: RoR above       */
-    { R"(\\spv3\.exe$)", {{
-      { "d3d9.shaderModel",                 "1" },
-    }} },
     /* Escape from Tarkov launcher
        Same issue as Warhammer: RoR above       */
     { R"(\\BsgLauncher\.exe$)", {{
@@ -708,6 +722,44 @@ namespace dxvk {
     { R"(\\BBCF\.exe$)", {{
       { "d3d9.textureMemory",               "0"   },
     }} },
+    /* Battle Fantasia Revised Edition         *
+     * Speedup above 60fps                     */
+    { R"(\\bf10\.exe$)", {{
+      { "d3d9.maxFrameRate",                "60" },
+    }} },
+    /* WILD HEARTS™️                            *
+     * D3D12 title using D3D11 device for      *
+     * media texture creation, whereby a large *
+     * chunk size only slows down media        *
+     * initialization                          */
+    { R"(\\WILD HEARTS(_Trial)?\.exe$)", {{
+      { "dxvk.maxChunkSize",                 "4" },
+    }} },
+    /* Codename Panzers Phase One/Two          *
+     * Main menu won't render after intros     */
+    { R"(\\(PANZERS|PANZERS_Phase_2)\.exe$)", {{
+      { "d3d9.enableDialogMode",         "True"   },
+    }} },
+    /* Dark Romance: Vampire in Love          *
+     * Works around black screen or blinking  */
+    { R"(\\(DarkRomance_VampireInLove_CE)\.exe$)", {{
+      { "d3d9.allowDirectBufferMapping", "False"   },
+    }} },
+    /* DC Universe Online                      *
+     * Freezes after alt tabbing               */
+    { R"(\\DCGAME\.EXE$)", {{
+      { "d3d9.deviceLost",              "True"   },
+    }} },
+    
+    /**********************************************/
+    /* D3D12 GAMES (vkd3d-proton with dxvk dxgi)  */
+    /**********************************************/
+    
+    /* Diablo 4 - Will complain about missing  *
+     * GPU unless dxgi Id match actual GPU Id  */
+    { R"(\\Diablo IV\.exe$)", {{
+      { "dxgi.nvapiHack",                "False"  },
+    }} },
   }};
 
 
